@@ -2,7 +2,7 @@
  * commit.c :  routines for committing changes to the server
  *
  * ====================================================================
- * Copyright (c) 2000-2007 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2008 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -26,8 +26,6 @@
 #define APR_WANT_STRFUNC
 #include <apr_want.h>
 
-#include <assert.h>
-
 #include "svn_pools.h"
 #include "svn_error.h"
 #include "svn_delta.h"
@@ -294,7 +292,6 @@ static svn_error_t * get_activity_collection(commit_ctx_t *cc,
   /* use our utility function to fetch the activity URL */
   SVN_ERR(svn_ra_neon__get_activity_collection(collection,
                                                cc->ras,
-                                               cc->ras->root.path,
                                                pool));
 
   if (cc->push_func != NULL)
@@ -634,10 +631,7 @@ static svn_error_t * commit_open_root(void *edit_baton,
   /* create the root resource. no wr_url (yet). */
   rsrc = apr_pcalloc(dir_pool, sizeof(*rsrc));
   rsrc->pool = dir_pool;
-
-  /* ### should this be 'base_revision' here? we might not always be
-     ### working against the head! (think "properties"). */
-  rsrc->revision = SVN_INVALID_REVNUM;
+  rsrc->revision = base_revision;
 
   rsrc->url = cc->ras->root.path;
   rsrc->local_path = "";
@@ -751,15 +745,11 @@ static svn_error_t * commit_delete_entry(const char *path,
                    APR_HASH_KEY_STRING, SVN_DAV_OPTION_KEEP_LOCKS);
     }
 
-  /* 404 is ignored, because mod_dav_svn is effectively merging
-     against the HEAD revision on-the-fly.  In such a universe, a
-     failed deletion (because it's already missing) is OK;  deletion
-     is an idempotent merge operation. */
   serr = svn_ra_neon__simple_request(&code, parent->cc->ras,
                                      "DELETE", child,
                                      extra_headers, NULL,
-                                     204 /* Created */,
-                                     404 /* Not Found */, pool);
+                                     204 /* No Content */,
+                                     0, pool);
 
   /* A locking-related error most likely means we were deleting a
      directory rather than a file, and didn't send all of the
@@ -978,9 +968,7 @@ static svn_error_t * commit_close_dir(void *dir_baton,
 
   /* Perform all of the property changes on the directory. Note that we
      checked out the directory when the first prop change was noted. */
-  SVN_ERR(do_proppatch(dir->cc->ras, dir->rsrc, dir, pool));
-
-  return SVN_NO_ERROR;
+  return do_proppatch(dir->cc->ras, dir->rsrc, dir, pool);
 }
 
 static svn_error_t * commit_add_file(const char *path,
@@ -1021,9 +1009,9 @@ static svn_error_t * commit_add_file(const char *path,
 
   /* If the parent directory existed before this commit then there may be a
      file with this URL already. We need to ensure such a file does not
-     exist, which we do by attempting a PROPFIND.  Of course, a
-     PROPFIND *should* succeed if this "add" is actually the second
-     half of a "replace".
+     exist, which we do by attempting a PROPFIND in both public URL (the path
+     in HEAD) and the working URL (the path within the transaction), since
+     we cannot differentiate between deleted items.
 
      ### For now, we'll assume that if this path has already been
      added to the valid targets hash, that addition occurred during the
@@ -1034,25 +1022,31 @@ static svn_error_t * commit_add_file(const char *path,
       && (! apr_hash_get(file->cc->valid_targets, path, APR_HASH_KEY_STRING)))
     {
       svn_ra_neon__resource_t *res;
-      svn_error_t *err = svn_ra_neon__get_starting_props(&res,
-                                                         file->cc->ras,
-                                                         file->rsrc->wr_url,
-                                                         NULL, workpool);
-      if (!err)
+      svn_error_t *err1 = svn_ra_neon__get_starting_props(&res,
+                                                          file->cc->ras,
+                                                          file->rsrc->wr_url,
+                                                          NULL, workpool);
+      svn_error_t *err2 = svn_ra_neon__get_starting_props(&res,
+                                                          file->cc->ras,
+                                                          file->rsrc->url,
+                                                          NULL, workpool);
+      if (! err1 && ! err2)
         {
-          /* If the PROPFIND succeeds the file already exists */
+          /* If the PROPFINDs succeed the file already exists */
           return svn_error_createf(SVN_ERR_RA_DAV_ALREADY_EXISTS, NULL,
                                    _("File '%s' already exists"),
                                    file->rsrc->url);
         }
-      else if (err->apr_err == SVN_ERR_RA_DAV_PATH_NOT_FOUND)
+      else if ((err1 && (err1->apr_err == SVN_ERR_FS_NOT_FOUND))
+               || (err2 && (err2->apr_err == SVN_ERR_FS_NOT_FOUND)))
         {
-          svn_error_clear(err);
+          svn_error_clear(err1);
+          svn_error_clear(err2);
         }
       else
         {
           /* A real error */
-          return err;
+          return svn_error_compose_create(err1, err2);
         }
     }
 
@@ -1188,6 +1182,8 @@ commit_apply_txdelta(void *file_baton,
   resource_baton_t *file = file_baton;
   put_baton_t *baton;
   svn_stream_t *stream;
+  const char *tempfile_name;
+  svn_checksum_t *checksum;
 
   baton = apr_pcalloc(file->pool, sizeof(*baton));
   baton->ras = file->cc->ras;
@@ -1203,20 +1199,28 @@ commit_apply_txdelta(void *file_baton,
      ### or a FILE*. since we are getting binary data, we must use a FILE*
      ### for now. isn't that special? */
 
-  /* Use the client callback to create a tmpfile. */
-  SVN_ERR(file->cc->ras->callbacks->open_tmp_file
-          (&baton->tmpfile,
-           file->cc->ras->callback_baton,
-           file->pool));
+  /* Create a temp file in the system area to hold the contents. Note that
+     we need a file since we will be rewinding it. The file will be closed
+     and deleted when the pool is cleaned up.  Avoid temp file name
+     collisions by requesting unique temp file name based on the checksum
+     of FILE->RSRC->LOCAL_PATH. */
+  SVN_ERR(svn_checksum(&checksum, svn_checksum_md5,
+                       file->rsrc->local_path,
+                       strlen(file->rsrc->local_path),
+                       pool));
+  tempfile_name = apr_psprintf(pool, "tempfile.%s",
+                               svn_checksum_to_cstring_display(checksum,
+                                                               pool));  
 
-  /* ### register a cleanup on file_pool which closes the file; this
-     ### will ensure that the file always gets tossed, even if we exit
-     ### with an error. */
+  SVN_ERR(svn_io_open_uniquely_named(&baton->tmpfile, NULL, NULL,
+                                     tempfile_name, ".tmp",
+                                     svn_io_file_del_on_pool_cleanup,
+                                     file->pool, pool));
 
   stream = svn_stream_create(baton, pool);
   svn_stream_set_write(stream, commit_stream_write);
 
-  svn_txdelta_to_svndiff(stream, pool, handler, handler_baton);
+  svn_txdelta_to_svndiff2(handler, handler_baton, stream, 0, pool);
 
   /* Add this path to the valid targets hash. */
   add_valid_target(file->cc, file->rsrc->local_path, svn_nonrecursive);
@@ -1319,16 +1323,16 @@ static svn_error_t * commit_close_file(void *file_baton,
 
       if (pb->tmpfile)
         {
-          /* we're done with the file.  this should delete it. */
+          /* We're done with the file.  this should delete it. Note: it
+             isn't a big deal if this line is never executed -- the pool
+             will eventually get it. We're just being proactive here. */
           (void) apr_file_close(pb->tmpfile);
         }
     }
 
   /* Perform all of the property changes on the file. Note that we
      checked out the file when the first prop change was noted. */
-  SVN_ERR(do_proppatch(cc->ras, file->rsrc, file, pool));
-
-  return SVN_NO_ERROR;
+  return do_proppatch(cc->ras, file->rsrc, file, pool);
 }
 
 
